@@ -1,7 +1,7 @@
 /*
  * frogr-config.c -- Configuration system for Frogr.
  *
- * Copyright (C) 2009-2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2012 Mario Sanchez Prada
  *           (C) 2009 Adrian Perez
  * Authors:
  *   Adrian Perez <aperez@igalia.com>
@@ -46,12 +46,15 @@ G_DEFINE_TYPE (FrogrConfig, frogr_config, G_TYPE_OBJECT)
 typedef struct _FrogrConfigPrivate FrogrConfigPrivate;
 struct _FrogrConfigPrivate
 {
+  gchar *config_dir;
+
   GSList *accounts;
   FrogrAccount *active_account;
 
   gboolean public;
   gboolean family;
   gboolean friend;
+  gboolean send_geolocation_data;
   gboolean show_in_search;
 
   FspLicense license;
@@ -60,9 +63,12 @@ struct _FrogrConfigPrivate
 
   gboolean tags_autocompletion;
   gboolean keep_file_extensions;
+  gboolean import_tags_from_metadata;
+
   SortingCriteria mainview_sorting_criteria;
   gboolean mainview_sorting_reversed;
   gboolean mainview_enable_tooltips;
+  gboolean use_dark_theme;
 
   gboolean use_proxy;
   gboolean use_gnome_proxy;
@@ -70,6 +76,8 @@ struct _FrogrConfigPrivate
   gchar *proxy_port;
   gchar *proxy_username;
   gchar *proxy_password;
+
+  gchar *settings_version;
 };
 
 static FrogrConfig *_instance = NULL;
@@ -78,7 +86,7 @@ static FrogrConfig *_instance = NULL;
 
 static FrogrAccount *_find_account_by_id (FrogrConfig *self, const gchar *id);
 
-static void _load_settings (FrogrConfig *self, const gchar *config_dir);
+static void _load_settings (FrogrConfig *self);
 
 static void _load_visibility_xml (FrogrConfig *self,
                                   xmlDocPtr     xml,
@@ -92,8 +100,7 @@ static void _load_proxy_data_xml (FrogrConfig *self,
                                   xmlDocPtr     xml,
                                   xmlNodePtr    rootnode);
 
-static void _load_accounts (FrogrConfig *self,
-                            const gchar *config_dir);
+static void _load_accounts (FrogrConfig *self);
 
 static gboolean _load_account_xml (FrogrAccount *faccount,
                                    xmlDocPtr     xml,
@@ -141,7 +148,7 @@ _find_account_by_id (FrogrConfig *self, const gchar *id)
 }
 
 static void
-_load_settings (FrogrConfig *self, const gchar *config_dir)
+_load_settings (FrogrConfig *self)
 {
   FrogrConfigPrivate *priv = NULL;
   gchar *xml_path = NULL;
@@ -149,21 +156,30 @@ _load_settings (FrogrConfig *self, const gchar *config_dir)
   xmlDocPtr xml = NULL;
 
   g_return_if_fail (FROGR_IS_CONFIG (self));
-  g_return_if_fail (config_dir != NULL);
 
   priv = FROGR_CONFIG_GET_PRIVATE (self);
 
-  xml_path = g_build_filename (config_dir, SETTINGS_FILENAME, NULL);
+  xml_path = g_build_filename (priv->config_dir, SETTINGS_FILENAME, NULL);
   if (g_file_test (xml_path, G_FILE_TEST_IS_REGULAR))
     xml = xmlParseFile (xml_path);
 
   if (xml)
     node = xmlDocGetRootElement (xml);
   else
-    DEBUG ("Could not load '%s/%s'", config_dir, SETTINGS_FILENAME);
+    DEBUG ("Could not load '%s/%s'", priv->config_dir, SETTINGS_FILENAME);
 
   if (node && node->name && !xmlStrcmp (node->name, (const xmlChar*) "settings"))
     {
+      xmlChar *version = NULL;
+
+      /* Check version of the settings file first */
+      version = xmlGetProp (node, (const xmlChar *) "version");
+      g_free (priv->settings_version);
+      priv->settings_version = g_strdup (version ? (gchar *) version : "1");
+
+      if (version)
+        xmlFree (version);
+
       /* Iterate over children nodes and extract accounts. */
       for (node = node->children; node != NULL; node = node->next)
         {
@@ -262,6 +278,16 @@ _load_settings (FrogrConfig *self, const gchar *config_dir)
               xmlFree (content);
             }
 
+          if (!xmlStrcmp (node->name, (const xmlChar*) "import-tags-from-metadata"))
+            {
+              xmlChar *content = NULL;
+
+              content = xmlNodeGetContent (node);
+              priv->import_tags_from_metadata = !xmlStrcmp (content, (const xmlChar*) "1");
+
+              xmlFree (content);
+            }
+
           if (!xmlStrcmp (node->name, (const xmlChar*) "mainview-options"))
             _load_mainview_options_xml (self, xml, node);
 
@@ -270,16 +296,26 @@ _load_settings (FrogrConfig *self, const gchar *config_dir)
 
           if (!xmlStrcmp (node->name, (const xmlChar*) "http-proxy"))
             _load_proxy_data_xml (self, xml, node);
+
+          if (!xmlStrcmp (node->name, (const xmlChar*) "use-dark-theme"))
+            {
+              xmlChar *content = NULL;
+
+              content = xmlNodeGetContent (node);
+              priv->use_dark_theme = !xmlStrcmp (content, (const xmlChar*) "1");
+
+              xmlFree (content);
+            }
         }
     }
   else if (node && node->name)
     {
       g_warning ("File '%s/%s' does not start with "
-                 "a <settings> tag", config_dir, SETTINGS_FILENAME);
+                 "a <settings> tag", priv->config_dir, SETTINGS_FILENAME);
     }
   else if (!node && xml)
     {
-      g_warning ("File '%s/%s' is empty", config_dir, SETTINGS_FILENAME);
+      g_warning ("File '%s/%s' is empty", priv->config_dir, SETTINGS_FILENAME);
     }
 
   if (xml)
@@ -330,6 +366,12 @@ _load_visibility_xml (FrogrConfig *self,
           priv->friend = !xmlStrcmp (content, (const xmlChar*) "1");
         }
 
+      if (!xmlStrcmp (node->name, (const xmlChar*) "send-geolocation-data"))
+        {
+          content = xmlNodeGetContent (node);
+          priv->send_geolocation_data = !xmlStrcmp (content, (const xmlChar*) "1");
+        }
+
       if (!xmlStrcmp (node->name, (const xmlChar*) "show-in-search"))
         {
           content = xmlNodeGetContent (node);
@@ -475,23 +517,24 @@ _load_proxy_data_xml (FrogrConfig *self,
 }
 
 static void
-_load_accounts (FrogrConfig *self, const gchar *config_dir)
+_load_accounts (FrogrConfig *self)
 {
+  FrogrConfigPrivate *priv = NULL;
   gchar *xml_path = NULL;
   xmlNodePtr node = NULL;
   xmlDocPtr xml = NULL;
 
   g_return_if_fail (FROGR_IS_CONFIG (self));
-  g_return_if_fail (config_dir != NULL);
 
-  xml_path = g_build_filename (config_dir, ACCOUNTS_FILENAME, NULL);
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  xml_path = g_build_filename (priv->config_dir, ACCOUNTS_FILENAME, NULL);
   if (g_file_test (xml_path, G_FILE_TEST_IS_REGULAR))
     xml = xmlParseFile (xml_path);
 
   if (xml)
     node = xmlDocGetRootElement (xml);
   else
-    DEBUG ("Could not load '%s/%s'", config_dir, ACCOUNTS_FILENAME);
+    DEBUG ("Could not load '%s/%s'", priv->config_dir, ACCOUNTS_FILENAME);
 
   if (node && node->name && !xmlStrcmp (node->name, (const xmlChar*) "accounts"))
     {
@@ -508,7 +551,7 @@ _load_accounts (FrogrConfig *self, const gchar *config_dir)
               else
                 {
                   g_warning ("Malformed account in '%s/%s', "
-                             "skipping it", config_dir, ACCOUNTS_FILENAME);
+                             "skipping it", priv->config_dir, ACCOUNTS_FILENAME);
 
                   xmlUnlinkNode (node);
                   xmlFreeNode (node);
@@ -523,11 +566,11 @@ _load_accounts (FrogrConfig *self, const gchar *config_dir)
   else if (node && node->name)
     {
       g_warning ("File '%s/%s' does not start with "
-                 "an <accounts> tag", config_dir, ACCOUNTS_FILENAME);
+                 "an <accounts> tag", priv->config_dir, ACCOUNTS_FILENAME);
     }
   else if (!node && xml)
     {
-      g_warning ("File '%s/%s' is empty", config_dir, ACCOUNTS_FILENAME);
+      g_warning ("File '%s/%s' is empty", priv->config_dir, ACCOUNTS_FILENAME);
     }
 
   if (xml)
@@ -543,11 +586,19 @@ _load_account_xml (FrogrAccount *faccount,
 {
   xmlNodePtr node = NULL;
   xmlChar *content = NULL;
+  xmlChar *version = NULL;
 
   g_return_val_if_fail (faccount != NULL, FALSE);
   g_return_val_if_fail (xml      != NULL, FALSE);
   g_return_val_if_fail (rootnode != NULL, FALSE);
 
+  /* Check version of the account first */
+  version = xmlGetProp (rootnode, (const xmlChar *) "version");
+  frogr_account_set_version (faccount, version ? (gchar *) version : "1");
+
+  if (version)
+    xmlFree (version);
+
   /* Traverse child nodes and extract relevant information. */
   for (node = rootnode->children; node != NULL; node = node->next)
     {
@@ -564,6 +615,13 @@ _load_account_xml (FrogrAccount *faccount,
             frogr_account_set_token (faccount, (gchar *)content);
         }
 
+      if (xmlStrcmp (node->name, (const xmlChar*) "token-secret") == 0)
+        {
+          content = xmlNodeGetContent (node);
+          if (content != NULL && content[0] != '\0')
+            frogr_account_set_token_secret (faccount, (gchar *)content);
+        }
+
       if (xmlStrcmp (node->name, (const xmlChar*) "permissions") == 0)
         {
           content = xmlNodeGetContent (node);
@@ -628,11 +686,15 @@ _save_settings (FrogrConfig *self)
   root = xmlNewNode (NULL, (const xmlChar*) "settings");
   xmlDocSetRootElement (xml, root);
 
+  /* Settings versioning */
+  xmlSetProp (root, (const xmlChar*) "version", (const xmlChar*) SETTINGS_CURRENT_VERSION);
+
   /* Default visibility */
   node = xmlNewNode (NULL, (const xmlChar*) "default-visibility");
   _xml_add_bool_child (node, "public", priv->public);
   _xml_add_bool_child (node, "family", priv->family);
   _xml_add_bool_child (node, "friend", priv->friend);
+  _xml_add_bool_child (node, "send-geolocation-data", priv->send_geolocation_data);
   _xml_add_bool_child (node, "show-in-search", priv->show_in_search);
   xmlAddChild (root, node);
 
@@ -646,6 +708,8 @@ _save_settings (FrogrConfig *self)
   /* Other stuff */
   _xml_add_bool_child (root, "tags-autocompletion", priv->tags_autocompletion);
   _xml_add_bool_child (root, "keep-file-extensions", priv->keep_file_extensions);
+  _xml_add_bool_child (root, "import-tags-from-metadata", priv->import_tags_from_metadata);
+  _xml_add_bool_child (root, "use-dark-theme", priv->use_dark_theme);
   node = xmlNewNode (NULL, (const xmlChar*) "mainview-options");
   _xml_add_bool_child (node, "enable-tooltips", priv->mainview_enable_tooltips);
   _xml_add_int_child (node, "sorting-criteria", priv->mainview_sorting_criteria);
@@ -663,8 +727,7 @@ _save_settings (FrogrConfig *self)
   _xml_add_bool_child (node, "use-gnome-proxy", priv->use_gnome_proxy);
   xmlAddChild (root, node);
 
-  xml_path = g_build_filename (g_get_user_config_dir (), APP_SHORTNAME,
-			       SETTINGS_FILENAME, NULL);
+  xml_path = g_build_filename (priv->config_dir, SETTINGS_FILENAME, NULL);
 
   if (xmlSaveFormatFileEnc (xml_path, xml, "UTF-8", 1) == -1) {
     g_critical ("Unable to open '%s' for saving", xml_path);
@@ -704,8 +767,7 @@ _save_accounts (FrogrConfig *self)
       _save_account_xml (account, root);
     }
 
-  xml_path = g_build_filename (g_get_user_config_dir (), APP_SHORTNAME,
-			       ACCOUNTS_FILENAME, NULL);
+  xml_path = g_build_filename (priv->config_dir, ACCOUNTS_FILENAME, NULL);
 
   if (xmlSaveFormatFileEnc (xml_path, xml, "UTF-8", 1) == -1) {
     g_critical ("Unable to open '%s' for saving", xml_path);
@@ -726,7 +788,17 @@ _save_account_xml (FrogrAccount *faccount, xmlNodePtr parent)
 
   node = xmlNewNode (NULL, (const xmlChar*) "account");
   if (faccount) {
-    _xml_add_string_child (node, "token", frogr_account_get_token (faccount));
+    const gchar *token = NULL;
+
+    /* Accounts versioning */
+    xmlSetProp (node, (const xmlChar*) "version", (const xmlChar*) frogr_account_get_version (faccount));
+
+    if ((token = frogr_account_get_token (faccount)))
+      _xml_add_string_child (node, "token", token);
+
+    if ((token = frogr_account_get_token_secret (faccount)))
+      _xml_add_string_child (node, "token-secret", token);
+
     _xml_add_string_child (node, "permissions", frogr_account_get_permissions (faccount));
     _xml_add_string_child (node, "id", frogr_account_get_id (faccount));
     _xml_add_string_child (node, "username", frogr_account_get_username (faccount));
@@ -842,10 +914,12 @@ _finalize (GObject *object)
 {
   FrogrConfigPrivate *priv = FROGR_CONFIG_GET_PRIVATE (object);
 
+  g_free (priv->config_dir);
   g_free (priv->proxy_host);
   g_free (priv->proxy_port);
   g_free (priv->proxy_username);
   g_free (priv->proxy_password);
+  g_free (priv->settings_version);
 
   /* Call superclass */
   G_OBJECT_CLASS (frogr_config_parent_class)->finalize (object);
@@ -892,6 +966,7 @@ frogr_config_init (FrogrConfig *self)
 
   priv = FROGR_CONFIG_GET_PRIVATE (self);
 
+  priv->config_dir = NULL;
   priv->active_account = NULL;
   priv->accounts = NULL;
 
@@ -899,47 +974,55 @@ frogr_config_init (FrogrConfig *self)
   priv->public = FALSE;
   priv->family = FALSE;
   priv->friend = FALSE;
+  priv->send_geolocation_data = FALSE;
   priv->show_in_search = TRUE;
   priv->license = FSP_LICENSE_NONE;
   priv->safety_level = FSP_SAFETY_LEVEL_SAFE;
   priv->content_type = FSP_CONTENT_TYPE_PHOTO;
   priv->tags_autocompletion = TRUE;
   priv->keep_file_extensions = FALSE;
+  priv->import_tags_from_metadata = TRUE;
   priv->mainview_sorting_criteria = SORT_AS_LOADED;
   priv->mainview_sorting_reversed = FALSE;
   priv->mainview_enable_tooltips = TRUE;
+  priv->use_dark_theme = TRUE;
   priv->use_proxy = FALSE;
   priv->use_gnome_proxy = FALSE;
   priv->proxy_host = NULL;
   priv->proxy_port = NULL;
   priv->proxy_username = NULL;
   priv->proxy_password = NULL;
+  priv->settings_version = NULL;
 
   /* Ensure that we have the config directory in place. */
   config_dir = g_build_filename (g_get_user_config_dir (), APP_SHORTNAME, NULL);
-  if (g_mkdir_with_parents (config_dir, 0777) != 0)
+  if (g_mkdir_with_parents (config_dir, 0777) == 0)
+    {
+      priv->config_dir = g_strdup (config_dir);
+
+      /* Load data */
+      _load_settings (self);
+      _load_accounts (self);
+
+      /* Make sure at least one account is active, despite of not having
+         the <active> node present, for backwards compatibility */
+      if (g_slist_length (priv->accounts) > 0 && !priv->active_account)
+        {
+          FrogrAccount *account = NULL;
+          account = FROGR_ACCOUNT (priv->accounts->data);
+
+          frogr_account_set_is_active (account, TRUE);
+          priv->active_account = account;
+
+          _save_accounts (self);
+        }
+    }
+  else
     {
       g_warning ("Could not create config directory '%s' (%s)",
                  config_dir, strerror (errno));
     }
 
-  /* Load data */
-  _load_settings (self, config_dir);
-  _load_accounts (self, config_dir);
-
-  /* Make sure at least one account is active, despite of not having
-     the <active> node present, for backwards compatibility */
-  if (g_slist_length (priv->accounts) > 0 && !priv->active_account)
-    {
-      FrogrAccount *account = NULL;
-      account = FROGR_ACCOUNT (priv->accounts->data);
-
-      frogr_account_set_is_active (account, TRUE);
-      priv->active_account = account;
-
-      _save_accounts (self);
-    }
-
   g_free (config_dir);
 }
 
@@ -1210,6 +1293,28 @@ frogr_config_get_default_content_type (FrogrConfig *self)
   return priv->content_type;
 }
 
+void
+frogr_config_set_default_send_geolocation_data (FrogrConfig *self, gboolean value)
+{
+  FrogrConfigPrivate * priv = NULL;
+
+  g_return_if_fail (FROGR_IS_CONFIG (self));
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  priv->send_geolocation_data = value;
+}
+
+gboolean
+frogr_config_get_default_send_geolocation_data (FrogrConfig *self)
+{
+  FrogrConfigPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_CONFIG (self), FALSE);
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  return priv->send_geolocation_data;
+}
+
 void
 frogr_config_set_default_show_in_search (FrogrConfig *self, gboolean value)
 {
@@ -1276,6 +1381,28 @@ frogr_config_get_keep_file_extensions (FrogrConfig *self)
   return priv->keep_file_extensions;
 }
 
+void
+frogr_config_set_import_tags_from_metadata (FrogrConfig *self, gboolean value)
+{
+  FrogrConfigPrivate * priv = NULL;
+
+  g_return_if_fail (FROGR_IS_CONFIG (self));
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  priv->import_tags_from_metadata = value;
+}
+
+gboolean
+frogr_config_get_import_tags_from_metadata (FrogrConfig *self)
+{
+  FrogrConfigPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_CONFIG (self), FALSE);
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  return priv->import_tags_from_metadata;
+}
+
 void
 frogr_config_set_mainview_enable_tooltips (FrogrConfig *self, gboolean value)
 {
@@ -1298,6 +1425,28 @@ frogr_config_get_mainview_enable_tooltips (FrogrConfig *self)
   return priv->mainview_enable_tooltips;
 }
 
+void
+frogr_config_set_use_dark_theme (FrogrConfig *self, gboolean value)
+{
+  FrogrConfigPrivate * priv = NULL;
+
+  g_return_if_fail (FROGR_IS_CONFIG (self));
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  priv->use_dark_theme = value;
+}
+
+gboolean
+frogr_config_get_use_dark_theme (FrogrConfig *self)
+{
+  FrogrConfigPrivate *priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_CONFIG (self), FALSE);
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  return priv->use_dark_theme;
+}
+
 void
 frogr_config_set_mainview_sorting_criteria (FrogrConfig *self,
                                             SortingCriteria criteria)
@@ -1489,3 +1638,14 @@ frogr_config_get_proxy_password (FrogrConfig *self)
   priv = FROGR_CONFIG_GET_PRIVATE (self);
   return priv->proxy_password;
 }
+
+const gchar *
+frogr_config_get_settings_version (FrogrConfig *self)
+{
+  FrogrConfigPrivate * priv = NULL;
+
+  g_return_val_if_fail (FROGR_IS_CONFIG (self), FALSE);
+
+  priv = FROGR_CONFIG_GET_PRIVATE (self);
+  return priv->settings_version;
+}
