@@ -1,7 +1,7 @@
 /*
  * frogr-details-dialog.c -- Picture details dialog
  *
- * Copyright (C) 2009-2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2012 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,6 +23,10 @@
 #include "frogr-config.h"
 #include "frogr-controller.h"
 #include "frogr-global-defs.h"
+#include "frogr-gtk-compat.h"
+#include "frogr-live-entry.h"
+#include "frogr-main-view.h"
+#include "frogr-main-view-model.h"
 #include "frogr-picture.h"
 #include "frogr-util.h"
 
@@ -54,6 +58,7 @@ typedef struct _FrogrDetailsDialogPrivate {
   GtkWidget *private_rb;
   GtkWidget *friend_cb;
   GtkWidget *family_cb;
+  GtkWidget *send_location_cb;
   GtkWidget *show_in_search_cb;
   GtkWidget *license_cb;
   GtkWidget *photo_content_rb;
@@ -70,7 +75,6 @@ typedef struct _FrogrDetailsDialogPrivate {
   GtkWidget *mpictures_label;
   GdkPixbuf *mpictures_pixbuf;
 
-  GtkTreeModel *treemodel;
   GSList *pictures;
   gulong picture_button_handler_id;
   gchar *reference_description;
@@ -82,11 +86,6 @@ enum  {
   PROP_PICTURES
 };
 
-/* Tree view columns */
-enum {
-  TEXT_COL,
-  N_COLS
-};
 
 static const gchar *license_descriptions[] = {
   N_("Default (as specified in flickr)"),
@@ -105,14 +104,6 @@ static const gchar *license_descriptions[] = {
 
 static void _create_widgets (FrogrDetailsDialog *self);
 
-static void _populate_treemodel_with_tags (FrogrDetailsDialog *self, GSList *tags);
-
-static gboolean _tag_list_completion_func (GtkEntryCompletion *completion, const gchar *key,
-                                           GtkTreeIter *iter, gpointer data);
-
-static gboolean _completion_match_selected_cb (GtkEntryCompletion *widget, GtkTreeModel *model,
-                                               GtkTreeIter *iter, gpointer data);
-
 static void _update_ui (FrogrDetailsDialog *self);
 
 static void _load_picture_from_disk (FrogrDetailsDialog *self);
@@ -144,7 +135,6 @@ static void
 _create_widgets (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv = NULL;
-  FrogrConfig *config = NULL;
   GtkWidget *main_vbox = NULL;
   GtkWidget *vbox = NULL;
   GtkWidget *hbox = NULL;
@@ -154,12 +144,12 @@ _create_widgets (FrogrDetailsDialog *self)
   GtkWidget *private_vbox = NULL;
   GtkWidget *content_type_hbox = NULL;
   GtkWidget *safety_level_hbox = NULL;
+
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
   GtkWidget *widget = NULL;
   GtkWidget *table = NULL;
   GtkWidget *scroller = NULL;
-  GtkTreeModel *model = NULL;
   gchar *markup = NULL;
   gint i;
 
@@ -167,13 +157,8 @@ _create_widgets (FrogrDetailsDialog *self)
 
   main_vbox = gtk_dialog_get_content_area (GTK_DIALOG (self));
 
-#ifdef GTK_API_VERSION_3
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-#else
-  hbox = gtk_hbox_new (FALSE, 0);
-  vbox = gtk_vbox_new (FALSE, 0);
-#endif
+  hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   /* Left side (image, radio buttons, checkboxes...) */
 
@@ -197,13 +182,8 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Visibility */
 
-#ifdef GTK_API_VERSION_3
-  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  visibility_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-#else
-  section_vbox = gtk_vbox_new (FALSE, 6);
-  visibility_vbox = gtk_vbox_new (FALSE, 6);
-#endif
+  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  visibility_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Visibility"));
@@ -215,11 +195,7 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
-#ifdef GTK_API_VERSION_3
-  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-#else
-  internal_hbox = gtk_hbox_new (FALSE, 6);
-#endif
+  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
   widget = gtk_radio_button_new_with_mnemonic (NULL, _("_Private"));
   gtk_box_pack_start (GTK_BOX (internal_hbox), widget, FALSE, FALSE, 0);
@@ -231,11 +207,7 @@ _create_widgets (FrogrDetailsDialog *self)
 
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
-#ifdef GTK_API_VERSION_3
-  private_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-#else
-  private_vbox = gtk_vbox_new (FALSE, 6);
-#endif
+  private_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
   widget = gtk_check_button_new_with_mnemonic (_("Visible to _Family"));
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
@@ -245,49 +217,25 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
   priv->friend_cb = widget;
 
-#ifdef GTK_API_VERSION_3
-  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#else
-  internal_hbox = gtk_hbox_new (FALSE, 0);
-#endif
+  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
   gtk_box_pack_start (GTK_BOX (internal_hbox), private_vbox, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
-#ifdef GTK_API_VERSION_3
-  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#else
-  internal_hbox = gtk_hbox_new (FALSE, 0);
-#endif
+  internal_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
   gtk_box_pack_start (GTK_BOX (internal_hbox), visibility_vbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (section_vbox), internal_hbox, FALSE, FALSE, 0);
 
-#ifdef GTK_API_VERSION_3
-  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#else
-  internal_hbox = gtk_hbox_new (FALSE, 0);
-#endif
-
-  widget = gtk_check_button_new_with_mnemonic (_("_Show up in Global Search Results"));
-  gtk_box_pack_start (GTK_BOX (internal_hbox), widget, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (section_vbox), internal_hbox, FALSE, FALSE, 0);
-  priv->show_in_search_cb = widget;
-
   gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 6);
 
   /* Content type */
 
-#ifdef GTK_API_VERSION_3
-  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  content_type_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-#else
-  section_vbox = gtk_vbox_new (FALSE, 6);
-  content_type_hbox = gtk_hbox_new (FALSE, 6);
-#endif
+  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  content_type_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-                                    _("Content type"));
+                                    _("Content Type"));
   widget = gtk_label_new (markup);
   gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
   g_free (markup);
@@ -314,16 +262,11 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Safety level */
 
-#ifdef GTK_API_VERSION_3
-  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  safety_level_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-#else
-  section_vbox = gtk_vbox_new (FALSE, 6);
-  safety_level_hbox = gtk_hbox_new (FALSE, 6);
-#endif
+  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  safety_level_hbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-                                    _("Safety level"));
+                                    _("Safety Level"));
   widget = gtk_label_new (markup);
   gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
   g_free (markup);
@@ -350,14 +293,10 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* License type */
 
-#ifdef GTK_API_VERSION_3
-  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-#else
-  section_vbox = gtk_vbox_new (FALSE, 6);
-#endif
+  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-                                    _("License type"));
+                                    _("License Type"));
   widget = gtk_label_new (markup);
   gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
   g_free (markup);
@@ -366,19 +305,39 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
-#if GTK_CHECK_VERSION (2,24,0)
-  widget = gtk_combo_box_text_new ();
+  widget = frogr_gtk_compat_combo_box_text_new ();
   for (i = 0; license_descriptions[i]; i++)
-    gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (widget), i, _(license_descriptions[i]));
-#else
-  for (i = 0; license_descriptions[i]; i++)
-    gtk_combo_box_insert_text (GTK_COMBO_BOX (widget), i, _(license_descriptions[i]));
-#endif
+    frogr_gtk_compat_combo_box_text_insert (GTK_COMBO_BOX (widget), i, _(license_descriptions[i]));
+
   priv->license_cb = widget;
   gtk_box_pack_start (GTK_BOX (section_vbox), widget, FALSE, FALSE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 6);
 
+  /* Other properties */
+
+  section_vbox = frogr_gtk_compat_box_new (GTK_ORIENTATION_VERTICAL, 6);
+
+  markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                    _("Other Properties"));
+  widget = gtk_label_new (markup);
+  gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
+  g_free (markup);
+
+  align = gtk_alignment_new (0, 0, 0, 0);
+  gtk_container_add (GTK_CONTAINER (align), widget);
+  gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
+
+  widget = gtk_check_button_new_with_mnemonic (_("Set Geo_location Information"));
+  gtk_box_pack_start (GTK_BOX (section_vbox), widget, FALSE, FALSE, 0);
+  priv->send_location_cb = widget;
+
+  widget = gtk_check_button_new_with_mnemonic (_("_Show Up in Global Search Results"));
+  gtk_box_pack_start (GTK_BOX (section_vbox), widget, FALSE, FALSE, 0);
+  priv->show_in_search_cb = widget;
+
+  gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 6);
+
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 6);
 
   /* Right side (text fields) */
@@ -429,7 +388,7 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_table_attach (GTK_TABLE (table), align, 0, 1, 2, 3,
                     0, 0, 6, 6);
 
-  widget = gtk_entry_new ();
+  widget = frogr_live_entry_new ();
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
   gtk_table_attach (GTK_TABLE (table), widget, 1, 2, 2, 3,
                     GTK_EXPAND | GTK_FILL, 0, 6, 6);
@@ -439,29 +398,6 @@ _create_widgets (FrogrDetailsDialog *self)
 
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, TRUE, TRUE, 6);
 
-  model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
-  priv->treemodel = model;
-
-  config = frogr_config_get_instance ();
-  if (config && frogr_config_get_tags_autocompletion (config))
-    {
-      GtkEntryCompletion *completion = NULL;
-
-      /* Prepare auto completion for tags */
-      completion = gtk_entry_completion_new ();
-      gtk_entry_completion_set_text_column (GTK_ENTRY_COMPLETION (completion), TEXT_COL);
-      gtk_entry_completion_set_inline_completion (GTK_ENTRY_COMPLETION (completion), TRUE);
-      gtk_entry_completion_set_match_func (GTK_ENTRY_COMPLETION (completion),
-                                           _tag_list_completion_func,
-                                           self, NULL);
-      gtk_entry_set_completion (GTK_ENTRY (priv->tags_entry), completion);
-
-      gtk_entry_completion_set_model (GTK_ENTRY_COMPLETION (completion), model);
-
-      g_signal_connect (G_OBJECT (completion), "match-selected",
-                        G_CALLBACK (_completion_match_selected_cb), self);
-    }
-
   /* Connect signals */
   g_signal_connect (G_OBJECT (priv->public_rb), "clicked",
                     G_CALLBACK (_on_radio_button_clicked), self);
@@ -475,6 +411,9 @@ _create_widgets (FrogrDetailsDialog *self)
   g_signal_connect (G_OBJECT (priv->friend_cb), "toggled",
                     G_CALLBACK (_on_toggle_button_toggled), self);
 
+  g_signal_connect (G_OBJECT (priv->send_location_cb), "toggled",
+                    G_CALLBACK (_on_toggle_button_toggled), self);
+
   g_signal_connect (G_OBJECT (priv->show_in_search_cb), "toggled",
                     G_CALLBACK (_on_toggle_button_toggled), self);
 
@@ -500,116 +439,6 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_widget_show_all (main_vbox);
 }
 
-static void
-_populate_treemodel_with_tags (FrogrDetailsDialog *self, GSList *tags)
-{
-  FrogrDetailsDialogPrivate *priv = NULL;
-  GSList *item = NULL;
-  gchar *tag = NULL;
-  GtkTreeIter iter;
-
-  priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
-
-  for (item = tags; item; item = g_slist_next (item))
-    {
-      tag = (gchar *) item->data;
-      gtk_list_store_append (GTK_LIST_STORE (priv->treemodel), &iter);
-      gtk_list_store_set (GTK_LIST_STORE (priv->treemodel), &iter,
-                          0, tag, -1);
-    }
-}
-
-static gboolean
-_tag_list_completion_func (GtkEntryCompletion *completion, const gchar *key,
-                           GtkTreeIter *iter, gpointer data)
-{
-  FrogrDetailsDialog *self = NULL;
-  FrogrDetailsDialogPrivate *priv = NULL;
-  gchar *stripped_entry_text = NULL;
-  gchar *basetext = NULL;
-  gchar *tag = NULL;
-  gchar *lc_basetext = NULL;
-  gchar *lc_tag = NULL;
-  gint cursor_pos = 0;
-  gboolean matches = FALSE;
-
-  self = FROGR_DETAILS_DIALOG (data);
-  priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
-  gtk_tree_model_get (priv->treemodel, iter, TEXT_COL, &tag, -1);
-
-  /* Do nothing if not a valid tag */
-  if (!tag)
-    return FALSE;
-
-  /* Do nothing if the cursor is not in the last position */
-  cursor_pos = gtk_editable_get_position (GTK_EDITABLE (priv->tags_entry));
-  if (cursor_pos < gtk_entry_get_text_length (GTK_ENTRY (priv->tags_entry)))
-    return FALSE;
-
-  /* Look for the last token in 'key' */
-  stripped_entry_text = gtk_editable_get_chars (GTK_EDITABLE (priv->tags_entry), 0, cursor_pos);
-  stripped_entry_text = g_strstrip (stripped_entry_text);
-  basetext = g_strrstr (stripped_entry_text, " ");
-  if (basetext)
-    basetext++;
-  else
-    basetext = stripped_entry_text;
-
-  /* Downcase everything and compare */
-  lc_basetext = g_utf8_strdown (basetext, -1);
-  lc_tag = g_utf8_strdown (tag, -1);
-  if (g_str_has_prefix (lc_tag, lc_basetext))
-    matches = TRUE;
-
-  g_free (stripped_entry_text);
-  g_free (tag);
-  g_free (lc_basetext);
-  g_free (lc_tag);
-
-  return matches;
-}
-
-static gboolean
-_completion_match_selected_cb (GtkEntryCompletion *widget, GtkTreeModel *model,
-                               GtkTreeIter *iter, gpointer data)
-{
-  FrogrDetailsDialog *self = NULL;
-  FrogrDetailsDialogPrivate *priv = NULL;
-  gchar *tag = NULL;
-  const gchar *entry_text = NULL;
-  const gchar *matching_text = NULL;
-  gchar *base_text = NULL;
-  gchar *new_text = NULL;
-  glong entry_text_len = 0;
-  glong matching_text_len = 0;
-
-  self = FROGR_DETAILS_DIALOG (data);
-  priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
-  gtk_tree_model_get (model, iter, TEXT_COL, &tag, -1);
-
-  entry_text = gtk_entry_get_text (GTK_ENTRY (priv->tags_entry));
-  matching_text = g_strrstr (entry_text, " ");
-  if (matching_text)
-    matching_text++;
-  else
-    matching_text = entry_text;
-
-  entry_text_len = gtk_entry_get_text_length (GTK_ENTRY (priv->tags_entry));
-  matching_text_len = g_utf8_strlen (matching_text, -1);
-
-  base_text = gtk_editable_get_chars (GTK_EDITABLE (priv->tags_entry), 0, entry_text_len - matching_text_len);
-  new_text = g_strdup_printf ("%s%s ", base_text, tag);
-
-  gtk_entry_set_text (GTK_ENTRY (priv->tags_entry), new_text);
-  gtk_editable_set_position (GTK_EDITABLE (priv->tags_entry), -1);
-
-  g_free (tag);
-  g_free (base_text);
-  g_free (new_text);
-
-  return TRUE;
-}
-
 static void
 _update_ui (FrogrDetailsDialog *self)
 {
@@ -709,6 +538,8 @@ _load_picture_from_disk_cb (GObject *object,
           /* Everything should be fine by now, show it */
           _place_picture_in_dialog_and_show (self);
         }
+
+      g_object_unref (pixbuf_loader);
     }
 
   /* Show error to the user and finalize dialog if needed */
@@ -767,6 +598,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   gboolean is_public_val = FALSE;
   gboolean is_friend_val = FALSE;
   gboolean is_family_val = FALSE;
+  gboolean send_location_val = FALSE;
   gboolean show_in_search_val = FALSE;
   gboolean license_inconsistent = FALSE;
   FspLicense license_val = FSP_LICENSE_NONE;
@@ -783,6 +615,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   is_public_val = frogr_picture_is_public (picture);
   is_friend_val = frogr_picture_is_friend (picture);
   is_family_val = frogr_picture_is_family (picture);
+  send_location_val = frogr_picture_send_location (picture);
   show_in_search_val = frogr_picture_show_in_search (picture);
   license_val = frogr_picture_get_license (picture);
   safety_level_val = frogr_picture_get_safety_level (picture);
@@ -797,6 +630,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       gboolean is_public = FALSE;
       gboolean is_friend = FALSE;
       gboolean is_family = FALSE;
+      gboolean send_location = FALSE;
       gboolean show_in_search = FALSE;
       FspLicense license = FSP_LICENSE_NONE;
       FspSafetyLevel safety_level = FSP_SAFETY_LEVEL_NONE;
@@ -817,6 +651,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       is_public = frogr_picture_is_public (picture);
       is_friend = frogr_picture_is_friend (picture);
       is_family = frogr_picture_is_family (picture);
+      send_location = frogr_picture_send_location (picture);
       show_in_search = frogr_picture_show_in_search (picture);
 
       /* License */
@@ -863,6 +698,11 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
           gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->friend_cb),
                                               is_friend_val != is_friend);
         }
+      if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->send_location_cb)))
+        {
+          gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->send_location_cb),
+                                              send_location_val != send_location);
+        }
       if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->show_in_search_cb)))
         {
           gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->show_in_search_cb),
@@ -914,6 +754,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       is_public_val = is_public;
       is_friend_val = is_friend;
       is_family_val = is_family;
+      send_location_val = send_location;
       show_in_search_val = show_in_search;
       license_val = license;
       content_type_val = content_type;
@@ -948,6 +789,8 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
                                 is_friend_val);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->family_cb),
                                 is_family_val);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->send_location_cb),
+                                send_location_val);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->show_in_search_cb),
                                 show_in_search_val);
 
@@ -1036,6 +879,7 @@ _save_data (FrogrDetailsDialog *self)
   gboolean is_public;
   gboolean is_friend;
   gboolean is_family;
+  gboolean send_location;
   gboolean show_in_search;
   FspLicense license;
   FspSafetyLevel safety_level;
@@ -1072,6 +916,8 @@ _save_data (FrogrDetailsDialog *self)
       is_family = FALSE;
     }
 
+  send_location =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->send_location_cb));
   show_in_search =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->show_in_search_cb));
 
@@ -1122,6 +968,8 @@ _save_data (FrogrDetailsDialog *self)
             frogr_picture_set_friend (picture, is_friend);
           if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->family_cb)))
             frogr_picture_set_family (picture, is_family);
+          if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->send_location_cb)))
+            frogr_picture_set_send_location (picture, send_location);
           if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->show_in_search_cb)))
             frogr_picture_set_show_in_search (picture, show_in_search);
 
@@ -1141,6 +989,17 @@ _save_data (FrogrDetailsDialog *self)
           /* Everything went fine */
           result = TRUE;
         }
+
+      /* Add tags to the model */
+      if (!g_str_equal (tags, ""))
+        {
+          FrogrMainView *mainview = NULL;
+          FrogrMainViewModel *model = NULL;
+
+          mainview = frogr_controller_get_main_view (frogr_controller_get_instance ());
+          model = frogr_main_view_get_model (mainview);
+          frogr_main_view_model_add_local_tags_from_string (model, tags);
+        }
     }
   else
     {
@@ -1287,12 +1146,6 @@ _frogr_details_dialog_dispose (GObject *object)
       priv->pictures = NULL;
     }
 
-  if (priv->treemodel)
-    {
-      g_object_unref (priv->treemodel);
-      priv->treemodel = NULL;
-    }
-
   G_OBJECT_CLASS(frogr_details_dialog_parent_class)->dispose (object);
 }
 
@@ -1336,7 +1189,6 @@ frogr_details_dialog_init (FrogrDetailsDialog *self)
   FrogrDetailsDialogPrivate *priv =
     FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
-  priv->treemodel = NULL;
   priv->pictures = NULL;
   priv->picture_button_handler_id = 0;
   priv->reference_description = NULL;
@@ -1362,7 +1214,7 @@ frogr_details_dialog_init (FrogrDetailsDialog *self)
 void
 frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures, GSList *tags)
 {
-  FrogrDetailsDialog *self = NULL;
+  FrogrConfig *config = NULL;
   GObject *new = NULL;
 
   new = g_object_new (FROGR_TYPE_DETAILS_DIALOG,
@@ -1375,8 +1227,15 @@ frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures, GSList *tags)
                       "title", _("Edit Picture Details"),
                       NULL);
 
-  self = FROGR_DETAILS_DIALOG (new);
+  /* Initialize values for widgets based on the data (pictures) passed */
+  _fill_dialog_with_data (FROGR_DETAILS_DIALOG (new));
 
-  _fill_dialog_with_data (self);
-  _populate_treemodel_with_tags (self, tags);
+  /* Enable tags autocompletion if needed */
+  config = frogr_config_get_instance ();
+  if (config && frogr_config_get_tags_autocompletion (config))
+    {
+      FrogrDetailsDialogPrivate *priv = NULL;
+      priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (new);
+      frogr_live_entry_set_auto_completion (FROGR_LIVE_ENTRY (priv->tags_entry), tags);
+    }
 }
